@@ -13,6 +13,7 @@ from osxphotos.debug import (
     set_debug,
     wrap_function,
 )
+from osxphotos.platform import is_macos
 
 # apply any debug functions
 # need to do this before importing anything else so that the debug functions
@@ -44,9 +45,7 @@ if args.get("--debug", False):
     print("Debugging enabled", file=sys.stderr)
 
 from .about import about
-from .add_locations import add_locations
 from .albums import albums
-from .batch_edit import batch_edit
 from .cli import cli_main
 from .cli_commands import (
     abort,
@@ -67,7 +66,6 @@ from .export import export
 from .exportdb import exportdb
 from .grep import grep
 from .help import help
-from .import_cli import import_cli
 from .info import info
 from .install_uninstall_run import install, run, uninstall
 from .keywords import keywords
@@ -76,27 +74,32 @@ from .labels import labels
 from .list import _list_libraries, list_libraries
 from .orphans import orphans
 from .persons import persons
-from .photo_inspect import photo_inspect
 from .places import places
 from .query import query
 from .repl import repl
-from .show_command import show
 from .snap_diff import diff, snap
-from .sync import sync
+from .template_repl import template_repl
 from .theme import theme
-from .timewarp import timewarp
 from .tutorial import tutorial
-from .uuid import uuid
 from .version import version
 
+if is_macos:
+    from .add_locations import add_locations
+    from .batch_edit import batch_edit
+    from .import_cli import import_cli
+    from .photo_inspect import photo_inspect
+    from .push_exif import push_exif
+    from .show_command import show
+    from .sync import sync
+    from .timewarp import timewarp
+    from .uuid import uuid
+
 install_traceback()
 
 __all__ = [
     "abort",
     "about",
-    "add_locations",
     "albums",
-    "batch_edit",
     "cli_main",
     "debug_dump",
     "diff",
@@ -109,7 +112,6 @@ __all__ = [
     "exportdb",
     "grep",
     "help",
-    "import_cli",
     "info",
     "install",
     "keywords",
@@ -120,7 +122,6 @@ __all__ = [
     "logger",
     "orphans",
     "persons",
-    "photo_inspect",
     "places",
     "query",
     "query_command",
@@ -128,9 +129,22 @@ __all__ = [
     "run",
     "selection_command",
     "set_debug",
-    "show",
     "snap",
+    "template_repl",
     "tutorial",
-    "uuid",
     "verbose",
+    "version",
 ]
+
+if is_macos:
+    __all__ += [
+        "add_locations",
+        "batch_edit",
+        "import_cli",
+        "photo_inspect",
+        "push_exif",
+        "show",
+        "sync",
+        "timewarp",
+        "uuid",
+    ]
