@@ -7,6 +7,7 @@ import yaml
 
 import osxphotos
 from osxphotos._constants import _PHOTOS_4_VERSION
+from osxphotos.iphoto import is_iphoto_library
 
 from .cli_params import DB_ARGUMENT, DB_OPTION, JSON_OPTION
 from .common import get_photos_db
@@ -31,7 +32,11 @@ def albums(ctx, cli_obj, db, json_, photos_library):
         _list_libraries()
         return
 
-    photosdb = osxphotos.PhotosDB(dbfile=db)
+    photosdb = (
+        osxphotos.iPhotoDB(db)
+        if is_iphoto_library(db)
+        else osxphotos.PhotosDB(dbfile=db)
+    )
     albums = {"albums": photosdb.albums_as_dict}
     if photosdb.db_version > _PHOTOS_4_VERSION:
         albums["shared albums"] = photosdb.albums_shared_as_dict
